@@ -60,7 +60,7 @@ enum
     PROP_SHELL,
 };
 
-G_DEFINE_ABSTRACT_TYPE (CcPanel, cc_panel, GTK_TYPE_ALIGNMENT)
+G_DEFINE_ABSTRACT_TYPE (CcPanel, cc_panel, GTK_TYPE_BIN)
 
 static void
 cc_panel_set_property (GObject      *object,
@@ -123,17 +123,65 @@ cc_panel_finalize (GObject *object)
   G_OBJECT_CLASS (cc_panel_parent_class)->finalize (object);
 }
 
+static void
+cc_panel_get_preferred_width (GtkWidget *widget,
+                              gint      *minimum,
+                              gint      *natural)
+{
+  GtkBin *bin = GTK_BIN (widget);
+  GtkWidget *child;
+
+  *minimum = *natural = 0;
+
+  if ((child = gtk_bin_get_child (bin)))
+    gtk_widget_get_preferred_width (child, minimum, natural);
+}
+
+static void
+cc_panel_get_preferred_height (GtkWidget *widget,
+                               gint      *minimum,
+                               gint      *natural)
+{
+  GtkBin *bin = GTK_BIN (widget);
+  GtkWidget *child;
+
+  *minimum = *natural = 0;
+
+  if ((child = gtk_bin_get_child (bin)))
+    gtk_widget_get_preferred_height (child, minimum, natural);
+}
+
+static void
+cc_panel_size_allocate (GtkWidget     *widget,
+                             GtkAllocation *allocation)
+{
+  GtkAllocation child_allocation;
+
+  gtk_widget_set_allocation (widget, allocation);
+
+  child_allocation = *allocation;
+
+  gtk_widget_size_allocate (gtk_bin_get_child (GTK_BIN (widget)),
+                            &child_allocation);
+}
 
 static void
 cc_panel_class_init (CcPanelClass *klass)
 {
   GParamSpec      *pspec;
   GObjectClass    *object_class = G_OBJECT_CLASS (klass);
+  GtkWidgetClass  *widget_class = GTK_WIDGET_CLASS (klass);
 
   object_class->get_property = cc_panel_get_property;
   object_class->set_property = cc_panel_set_property;
   object_class->finalize = cc_panel_finalize;
 
+  widget_class->get_preferred_width = cc_panel_get_preferred_width;
+  widget_class->get_preferred_height = cc_panel_get_preferred_height;
+  widget_class->size_allocate = cc_panel_size_allocate;
+
+  gtk_container_class_handle_border_width (GTK_CONTAINER_CLASS (klass));
+
   g_type_class_add_private (klass, sizeof (CcPanelPrivate));
 
   pspec = g_param_spec_object ("shell",
