@@ -20,9 +20,11 @@
  */
 
 #include <config.h>
-#include <gtk/gtk.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/wait.h>
+
+#include <gtk/gtk.h>
 #include "scrollarea.h"
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnomeui/gnome-rr.h>
@@ -31,7 +33,6 @@
 #include <gdk/gdkx.h>
 #include <X11/Xlib.h>
 #include <glib/gi18n.h>
-#include <gconf/gconf-client.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-bindings.h>
 
@@ -58,8 +59,9 @@ struct App
     GtkWidget	   *resolution_combo;
     GtkWidget	   *refresh_combo;
     GtkWidget	   *rotation_combo;
-    GtkWidget	   *panel_checkbox;
     GtkWidget	   *clone_checkbox;
+    GtkWidget	   *primary_display_checkbox;
+    GtkWidget	   *clone_label;
     GtkWidget	   *show_icon_checkbox;
 
     /* We store the event timestamp when the Apply button is clicked */
@@ -68,7 +70,6 @@ struct App
 
     GtkWidget      *area;
     gboolean	    ignore_gui_changes;
-    GConfClient	   *client;
 
     /* These are used while we are waiting for the ApplyConfiguration method to be executed over D-bus */
     DBusGConnection *connection;
@@ -81,11 +82,6 @@ struct App
     } apply_configuration_state;
 };
 
-/* Response codes for custom buttons in the main dialog */
-enum {
-    RESPONSE_MAKE_DEFAULT = 1
-};
-
 static void rebuild_gui (App *app);
 static void on_clone_changed (GtkWidget *box, gpointer data);
 static void on_rate_changed (GtkComboBox *box, gpointer data);
@@ -495,6 +491,7 @@ rebuild_mirror_screens (App *app)
     
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->clone_checkbox), mirror_is_active);
     gtk_widget_set_sensitive (app->clone_checkbox, mirror_is_supported);
+    gtk_widget_set_sensitive (app->clone_label, mirror_is_supported);
 
     g_signal_handlers_unblock_by_func (app->clone_checkbox, G_CALLBACK (on_clone_changed), app);
 }
@@ -589,7 +586,45 @@ rebuild_on_off_radios (App *app)
 static char *
 make_resolution_string (int width, int height)
 {
-    return g_strdup_printf (_("%d x %d"), width, height);
+    int ratio;
+    const char *aspect = NULL;
+
+    if (width && height) {
+	if (width > height)
+	    ratio = width * 10 / height;
+	else
+	    ratio = height * 10 / width;
+
+	switch (ratio) {
+	case 13:
+	    aspect = "4:3";
+            break;
+        case 16:
+            aspect = "16:10";
+            break;
+        case 17:
+            aspect = "16:9";
+            break;
+        case 12:
+            aspect = "5:4";
+            break;
+	/* This catches 1.5625 as well (1600x1024) when maybe it shouldn't. */
+        case 15:
+            aspect = "3:2";
+            break;
+        case 18:
+            aspect = "9:5";
+            break;
+        case 10:
+            aspect = "1:1";
+            break;
+	}
+    }
+
+    if (aspect != NULL)
+        return g_strdup_printf (_("%d x %d (%s)"), width, height, aspect);
+    else
+        return g_strdup_printf (_("%d x %d"), width, height);
 }
 
 static void
@@ -688,7 +723,7 @@ rebuild_gui (App *app)
 #if 0
     g_debug ("sensitive: %d, on: %d", sensitive, app->current_output->on);
 #endif
-    gtk_widget_set_sensitive (app->panel_checkbox, sensitive);
+    gtk_widget_set_sensitive (app->primary_display_checkbox, sensitive);
 
     app->ignore_gui_changes = FALSE;
 }
@@ -1386,6 +1421,8 @@ output_overlaps (GnomeOutputInfo *output, GnomeRRConfig *config)
     int i;
     GdkRectangle output_rect;
 
+    g_assert (output != NULL);
+
     get_output_rect (output, &output_rect);
 
     for (i = 0; config->outputs[i]; ++i)
@@ -1647,7 +1684,8 @@ static PangoLayout *
 get_display_name (App *app,
 		  GnomeOutputInfo *output)
 {
-    const char *text;
+    PangoLayout *layout;
+    char *text;
 
     if (app->current_configuration->clone) {
 	/* Translators:  this is the feature where what you see on your laptop's
@@ -1655,12 +1693,20 @@ get_display_name (App *app,
 	 * used as an adjective, not as a verb.  For example, the Spanish
 	 * translation could be "Pantallas en Espejo", *not* "Espejar Pantallas".
 	 */
-	text = _("Mirror Screens");
-    } else
-	text = output->display_name;
+	text = g_strdup(_("Mirror Screens"));
+    } else {
+        char *resolution;
 
-    return gtk_widget_create_pango_layout (
-	GTK_WIDGET (app->area), text);
+        resolution = make_resolution_string (output->width, output->height);
+        text = g_strdup_printf ("%s\n%s", output->display_name, resolution);
+        g_free (resolution);
+    }
+
+    layout = gtk_widget_create_pango_layout (GTK_WIDGET (app->area), text);
+    g_free (text);
+    pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
+
+    return layout;
 }
 
 static void
@@ -1677,9 +1723,9 @@ paint_background (FooScrollArea *area,
     widget_style = gtk_widget_get_style (widget);
 
     cairo_set_source_rgb (cr,
-                          widget_style->base[GTK_STATE_SELECTED].red / 65535.0,
-                          widget_style->base[GTK_STATE_SELECTED].green / 65535.0,
-                          widget_style->base[GTK_STATE_SELECTED].blue / 65535.0);
+                          widget_style->mid[GTK_STATE_NORMAL].red / 65535.0,
+                          widget_style->mid[GTK_STATE_NORMAL].green / 65535.0,
+                          widget_style->mid[GTK_STATE_NORMAL].blue / 65535.0);
 
     cairo_rectangle (cr,
 		     viewport.x, viewport.y,
@@ -1690,9 +1736,9 @@ paint_background (FooScrollArea *area,
     foo_scroll_area_add_input_from_fill (area, cr, on_canvas_event, NULL);
 
     cairo_set_source_rgb (cr,
-                          widget_style->dark[GTK_STATE_SELECTED].red / 65535.0,
-                          widget_style->dark[GTK_STATE_SELECTED].green / 65535.0,
-                          widget_style->dark[GTK_STATE_SELECTED].blue / 65535.0);
+                          widget_style->dark[GTK_STATE_NORMAL].red / 65535.0,
+                          widget_style->dark[GTK_STATE_NORMAL].green / 65535.0,
+                          widget_style->dark[GTK_STATE_NORMAL].blue / 65535.0);
 
     cairo_stroke (cr);
 }
@@ -1717,7 +1763,6 @@ paint_output (App *app, cairo_t *cr, int i)
     cairo_save (cr);
 
     foo_scroll_area_get_viewport (FOO_SCROLL_AREA (app->area), &viewport);
-
     get_geometry (output, &w, &h);
 
 #if 0
@@ -1739,8 +1784,6 @@ paint_output (App *app, cairo_t *cr, int i)
     g_debug ("%f %f %f %f", x, y, w * scale + 0.5, h * scale + 0.5);
 #endif
 
-    cairo_save (cr);
-
     cairo_translate (cr,
 		     x + (w * scale + 0.5) / 2,
 		     y + (h * scale + 0.5) / 2);
@@ -1757,7 +1800,6 @@ paint_output (App *app, cairo_t *cr, int i)
 		     - x - (w * scale + 0.5) / 2,
 		     - y - (h * scale + 0.5) / 2);
 
-
     cairo_rectangle (cr, x, y, w * scale + 0.5, h * scale + 0.5);
     cairo_clip_preserve (cr);
 
@@ -1827,8 +1869,6 @@ paint_output (App *app, cairo_t *cr, int i)
 static void
 on_area_paint (FooScrollArea  *area,
 	       cairo_t	      *cr,
-	       GdkRectangle   *extent,
-	       cairo_region_t *region,
 	       gpointer	       data)
 {
     App *app = data;
@@ -2150,19 +2190,6 @@ on_detect_displays (GtkWidget *widget, gpointer data)
     }
 }
 
-#define SHOW_ICON_KEY "/apps/gnome_settings_daemon/xrandr/show_notification_icon"
-
-
-static void
-on_show_icon_toggled (GtkWidget *widget, gpointer data)
-{
-    GtkToggleButton *tb = GTK_TOGGLE_BUTTON (widget);
-    App *app = data;
-
-    gconf_client_set_bool (app->client, SHOW_ICON_KEY,
-			   gtk_toggle_button_get_active (tb), NULL);
-}
-
 static GnomeOutputInfo *
 get_nearest_output (GnomeRRConfig *configuration, int x, int y)
 {
@@ -2261,6 +2288,15 @@ get_output_for_window (GnomeRRConfig *configuration, GdkWindow *window)
 				   win_rect.y + win_rect.height / 2);
 }
 
+static void
+on_toplevel_realized (GtkWidget *widget,
+                      App       *app)
+{
+  app->current_output = get_output_for_window (app->current_configuration,
+                                               gtk_widget_get_window (widget));
+  rebuild_gui (app);
+}
+
 /* We select the current output, i.e. select the one being edited, based on
  * which output is showing the configuration dialog.
  */
@@ -2271,13 +2307,14 @@ select_current_output_from_dialog_position (App *app)
 
     toplevel = gtk_widget_get_toplevel (app->panel);
 
-    if (gtk_widget_get_realized (toplevel))
+    if (gtk_widget_get_realized (toplevel)) {
 	app->current_output = get_output_for_window (app->current_configuration,
                                                      gtk_widget_get_window (toplevel));
-    else
+        rebuild_gui (app);
+    } else {
+        g_signal_connect (toplevel, "realize", G_CALLBACK (on_toplevel_realized), app);
 	app->current_output = NULL;
-
-    rebuild_gui (app);
+    }
 }
 
 /* This is a GtkWidget::map-event handler.  We wait for the display-properties
@@ -2307,64 +2344,6 @@ apply_button_clicked_cb (GtkButton *button, gpointer data)
     app->apply_button_clicked_timestamp = gtk_get_current_event_time ();
 }
 
-static void
-success_dialog_for_make_default (App *app)
-{
-    GtkWidget *dialog;
-    GtkWidget *toplevel;
-
-    toplevel = gtk_widget_get_toplevel (app->panel);
-
-    dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-				     GTK_DIALOG_MODAL,
-				     GTK_MESSAGE_INFO,
-				     GTK_BUTTONS_OK,
-				     _("The monitor configuration has been saved"));
-    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-					      _("This configuration will be used the next time someone logs in."));
-
-    gtk_dialog_run (GTK_DIALOG (dialog));
-    gtk_widget_destroy (dialog);
-}
-
-static void
-make_default (App *app)
-{
-    char *command_line;
-    char *source_filename;
-    char *dest_filename;
-    char *dest_basename;
-    GError *error;
-
-    if (!sanitize_and_save_configuration (app))
-	return;
-
-    dest_filename = gconf_client_get_string (app->client, "/apps/gnome_settings_daemon/xrandr/default_configuration_file", NULL);
-    if (!dest_filename)
-	return; /* FIXME: present an error? */
-
-    dest_basename = g_path_get_basename (dest_filename);
-
-    source_filename = gnome_rr_config_get_intended_filename ();
-
-    command_line = g_strdup_printf ("pkexec %s/gnome-display-properties-install-systemwide %s %s",
-				    SBINDIR,
-				    source_filename,
-				    dest_basename);
-
-    error = NULL;
-    /* FIXME: pick up stderr and present it nicely in case of error */
-    if (!g_spawn_command_line_sync (command_line, NULL, NULL, NULL, &error))
-	error_message (app, _("Could not set the default configuration for monitors"), error ? error->message : NULL);
-    else
-	success_dialog_for_make_default (app);
-
-    g_free (dest_filename);
-    g_free (dest_basename);
-    g_free (source_filename);
-    g_free (command_line);
-}
-
 static GtkWidget*
 _gtk_builder_get_widget (GtkBuilder *builder, const gchar *name)
 {
@@ -2375,7 +2354,6 @@ static void
 destroy_app (App *app)
 {
     gnome_rr_screen_destroy (app->screen);
-    g_object_unref (app->client);
     g_object_unref (app->builder);
     gnome_rr_labeler_hide (app->labeler);
     g_object_unref (app->labeler);
@@ -2419,13 +2397,11 @@ run_application (void)
 	return NULL;
     }
 
-    app->client = gconf_client_get_default ();
-
     app->panel = _gtk_builder_get_widget (builder, "display-panel");
 
     if (!app->panel)
       g_warning ("Missing display-panel object");
-    g_signal_connect_after (app->panel, "map-event",
+    g_signal_connect_after (app->panel, "show",
 			    G_CALLBACK (dialog_map_event_cb), app);
 
     app->current_monitor_event_box = _gtk_builder_get_widget (builder,
@@ -2459,24 +2435,18 @@ run_application (void)
     g_signal_connect (app->clone_checkbox, "toggled",
 		      G_CALLBACK (on_clone_changed), app);
 
+    app->clone_label    = _gtk_builder_get_widget (builder, "clone_resolution_warning_label");
+
     g_signal_connect (_gtk_builder_get_widget (builder, "detect_displays_button"),
 		      "clicked", G_CALLBACK (on_detect_displays), app);
 
-    app->show_icon_checkbox = _gtk_builder_get_widget (builder,
-						      "show_notification_icon");
-
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->show_icon_checkbox),
-				  gconf_client_get_bool (app->client, SHOW_ICON_KEY, NULL));
-
-    g_signal_connect (app->show_icon_checkbox, "toggled", G_CALLBACK (on_show_icon_toggled), app);
-
-    app->panel_checkbox = _gtk_builder_get_widget (builder, "panel_checkbox");
+    app->primary_display_checkbox = _gtk_builder_get_widget (builder, "primary_display_checkbox");
 
     make_text_combo (app->resolution_combo, 4);
     make_text_combo (app->refresh_combo, 3);
     make_text_combo (app->rotation_combo, -1);
 
-    g_assert (app->panel_checkbox);
+    g_assert (app->primary_display_checkbox);
 
     /* Scroll Area */
     app->area = (GtkWidget *)foo_scroll_area_new ();
@@ -2505,9 +2475,6 @@ run_application (void)
 
     g_signal_connect_swapped (_gtk_builder_get_widget (builder, "apply_button"),
                               "clicked", G_CALLBACK (apply), app);
-    g_signal_connect_swapped (_gtk_builder_get_widget (builder,
-                                                       "make_default_button"),
-                              "clicked", G_CALLBACK (make_default), app);
 
     g_object_weak_ref (G_OBJECT (app->panel), (GWeakNotify) destroy_app, app);
 
