@@ -28,34 +28,46 @@
 #include <gtk/gtk.h>
 #include <string.h>
 
-static void
-application_prepare_action_cb (GApplication       *application,
-                               GVariant           *arguments,
-                               GVariant           *platform_data,
-                               GnomeControlCenter *shell)
+
+static int
+application_command_line_cb (GApplication  *application,
+			     GApplicationCommandLine  *command_line,
+			     GnomeControlCenter      *shell)
 {
-  const gchar **argv;
-  gsize length;
+  int argc;
+  char **argv;
+  int retval = 0;
 
-  gnome_control_center_present (shell);
-
-  argv = g_variant_get_bytestring_array (arguments, &length);
-
-  if (length == 2)
+  argv = g_application_command_line_get_arguments (command_line, &argc);
+  if (argc == 2)
     {
+      gchar *start_id;
       GError *err = NULL;
-      const gchar *id = argv[1];
 
-      if (!cc_shell_set_active_panel_from_id (CC_SHELL (shell), id, &err))
+      start_id = argv[1];
+
+      if (!cc_shell_set_active_panel_from_id (CC_SHELL (shell), start_id, &err))
         {
+          g_warning ("Could not load setting panel \"%s\": %s", start_id,
+                     (err) ? err->message : "Unknown error");
+          retval = 1;
           if (err)
             {
-              g_warning ("Could not load setting panel \"%s\": %s", id,
-                         err->message);
               g_error_free (err);
+              err = NULL;
             }
         }
     }
+  g_strfreev (argv);
+  return retval;
+}
+
+static void
+application_startup_cb (GApplication       *application,
+			GnomeControlCenter *shell)
+{
+  gnome_control_center_show (shell, GTK_APPLICATION (application));
+  gnome_control_center_present (shell);
 }
 
 int
@@ -63,6 +75,7 @@ main (int argc, char **argv)
 {
   GnomeControlCenter *shell;
   GtkApplication *application;
+  int status;
 
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -75,33 +88,15 @@ main (int argc, char **argv)
   shell = gnome_control_center_new ();
 
   /* enforce single instance of this application */
-  application = gtk_application_new ("org.gnome.ControlCenter", &argc, &argv);
-  g_signal_connect (application, "prepare-activation",
-                    G_CALLBACK (application_prepare_action_cb), shell);
+  application = gtk_application_new ("org.gnome.ControlCenter", G_APPLICATION_HANDLES_COMMAND_LINE);
+  g_signal_connect (application, "startup",
+                    G_CALLBACK (application_startup_cb), shell);
+  g_signal_connect (application, "command-line",
+                    G_CALLBACK (application_command_line_cb), shell);
 
-  if (argc == 2)
-    {
-      gchar *start_id;
-      GError *err = NULL;
+  status = g_application_run (G_APPLICATION (application), argc, argv);
 
-      start_id = argv[1];
-
-      if (!cc_shell_set_active_panel_from_id (CC_SHELL (shell), start_id, &err))
-        {
-          g_warning ("Could not load setting panel \"%s\": %s", start_id,
-                     (err) ? err->message : "Unknown error");
-          if (err)
-            {
-              g_error_free (err);
-              err = NULL;
-            }
-        }
-    }
-
-  gtk_application_run (application);
-
-  g_object_unref (shell);
   g_object_unref (application);
 
-  return 0;
+  return status;
 }
