@@ -56,6 +56,8 @@ struct _GnomeControlCenterPrivate
 {
   GtkBuilder *builder;
   GtkWidget  *notebook;
+  GtkWidget  *main_vbox;
+  GtkWidget  *scrolled_window;
   GtkWidget  *window;
   GtkWidget  *search_entry;
 
@@ -69,77 +71,120 @@ struct _GnomeControlCenterPrivate
   guint32 last_time;
 
   GIOExtensionPoint *extension_point;
+
+  gchar *default_window_title;
+  gchar *default_window_icon;
 };
 
+/* Use a fixed width for the shell, since resizing horizontally is more awkward
+ * for the user than resizing vertically */
+#define FIXED_WIDTH 675
+
 
 static void
 activate_panel (GnomeControlCenter *shell,
                 const gchar        *id,
-                const gchar        *desktop_file)
+                const gchar        *desktop_file,
+                const gchar        *name,
+                const gchar        *icon_name)
 {
   GnomeControlCenterPrivate *priv = shell->priv;
   GAppInfo *appinfo;
   GError *err = NULL;
   GdkAppLaunchContext *ctx;
-  GKeyFile *key_file;
   GType panel_type = G_TYPE_INVALID;
   GList *panels, *l;
+  GKeyFile *key_file;
 
   /* check if there is an plugin that implements this panel */
   panels = g_io_extension_point_get_extensions (priv->extension_point);
 
-  for (l = panels; l != NULL; l = l->next)
+  if (!desktop_file)
+    return;
+
+  if (id)
     {
-      GIOExtension *extension;
-      const gchar *name;
 
-      extension = l->data;
-
-      name = g_io_extension_get_name (extension);
-
-      if (!g_strcmp0 (name, id))
+      for (l = panels; l != NULL; l = l->next)
         {
-          panel_type = g_io_extension_get_type (extension);
-          break;
+          GIOExtension *extension;
+          const gchar *name;
+
+          extension = l->data;
+
+          name = g_io_extension_get_name (extension);
+
+          if (!g_strcmp0 (name, id))
+            {
+              panel_type = g_io_extension_get_type (extension);
+              break;
+            }
         }
-    }
 
-  if (panel_type != G_TYPE_INVALID)
-    {
-      GtkWidget *panel;
-      gint i;
+      if (panel_type != G_TYPE_INVALID)
+        {
+          GtkWidget *panel;
+          GtkWidget *box;
+          gint i;
+          int nat_height;
 
-      /* create the panel plugin */
-      panel = g_object_new (panel_type, "shell", shell, NULL);
+          /* create the panel plugin */
+          panel = g_object_new (panel_type, "shell", shell, NULL);
 
-      /* switch to the new panel */
-      i = gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), panel, NULL);
-      gtk_widget_show_all (panel);
-      gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), i);
+          box = gtk_alignment_new (0, 0, 1, 1);
+          gtk_alignment_set_padding (GTK_ALIGNMENT (box), 6, 6, 6, 6);
 
-      return;
+          gtk_container_add (GTK_CONTAINER (box), panel);
+
+          i = gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), box,
+                                        NULL);
+
+          /* switch to the new panel */
+          gtk_widget_show (box);
+          gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), i);
+
+          /* set the title of the window */
+          gtk_window_set_title (GTK_WINDOW (priv->window), name);
+          gtk_window_set_icon_name (GTK_WINDOW (priv->window), icon_name);
+
+          gtk_widget_show (panel);
+
+          /* set the scrolled window small so that it doesn't force
+             the window to be larger than this panel */
+          gtk_scrolled_window_set_min_content_height (GTK_SCROLLED_WINDOW (priv->scrolled_window), 50);
+
+          /* resize to the preferred size of the panel */
+          gtk_widget_set_size_request (priv->window, FIXED_WIDTH, -1);
+          gtk_widget_get_preferred_height (GTK_WIDGET (priv->window),
+                                           NULL, &nat_height);
+          g_debug ("Setting panel height: %d", nat_height);
+          gtk_window_resize (GTK_WINDOW (priv->window),
+                             FIXED_WIDTH,
+                             nat_height);
+          return;
+        }
     }
 
 
   /* if a plugin was not found, then start app directly */
-  if (!desktop_file)
-    return;
+
+  if (err)
+    {
+      g_warning ("Error starting \"%s\": %s", desktop_file, err->message);
+
+      g_error_free (err);
+      err = NULL;
+
+      return;
+    }
 
   key_file = g_key_file_new ();
   g_key_file_load_from_file (key_file, desktop_file, 0, &err);
 
-  if (err)
-    {
-      g_warning ("Error starting \"%s\": %s", id, err->message);
-
-      g_error_free (err);
-      err = NULL;
-      return;
-    }
-
   appinfo = (GAppInfo*) g_desktop_app_info_new_from_keyfile (key_file);
 
   g_key_file_free (key_file);
+  key_file = NULL;
 
 
   ctx = gdk_app_launch_context_new ();
@@ -153,7 +198,7 @@ activate_panel (GnomeControlCenter *shell,
 
   if (err)
     {
-      g_warning ("Error starting \"%s\": %s", id, err->message);
+      g_warning ("Error starting \"%s\": %s", desktop_file, err->message);
       g_error_free (err);
       err = NULL;
     }
@@ -166,19 +211,17 @@ shell_show_overview_page (GnomeControlCenterPrivate *priv)
 
   gtk_notebook_remove_page (GTK_NOTEBOOK (priv->notebook), CAPPLET_PAGE);
 
-  gtk_label_set_text (GTK_LABEL (gtk_builder_get_object (priv->builder,
-                                                         "label-title")), "");
-  gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (priv->builder,
-                                                       "title-alignment")));
-
   /* clear the search text */
   g_free (priv->filter_string);
   priv->filter_string = g_strdup ("");
   gtk_entry_set_text (GTK_ENTRY (priv->search_entry), "");
+
+  /* reset window title and icon */
+  gtk_window_set_title (GTK_WINDOW (priv->window), priv->default_window_title);
+  gtk_window_set_icon_name (GTK_WINDOW (priv->window),
+                            priv->default_window_icon);
 }
 
-
-
 static void
 item_activated_cb (CcShellCategoryView *view,
                    gchar               *name,
@@ -199,6 +242,156 @@ item_activated_cb (CcShellCategoryView *view,
     }
 }
 
+static gboolean
+category_focus_out (GtkWidget          *view,
+                    GdkEventFocus      *event,
+                    GnomeControlCenter *shell)
+{
+  gtk_icon_view_unselect_all (GTK_ICON_VIEW (view));
+
+  return FALSE;
+}
+
+static gboolean
+category_focus_in (GtkWidget          *view,
+                   GdkEventFocus      *event,
+                   GnomeControlCenter *shell)
+{
+  GtkTreePath *path;
+
+  if (!gtk_icon_view_get_cursor (GTK_ICON_VIEW (view), &path, NULL))
+    {
+      path = gtk_tree_path_new_from_indices (0, -1);
+      gtk_icon_view_set_cursor (GTK_ICON_VIEW (view), path, NULL, FALSE);
+    }
+
+  gtk_icon_view_select_path (GTK_ICON_VIEW (view), path);
+  gtk_tree_path_free (path);
+
+  return FALSE;
+}
+
+static GList *
+get_item_views (GnomeControlCenter *shell)
+{
+  GList *list, *l;
+  GList *res;
+
+  list = gtk_container_get_children (GTK_CONTAINER (shell->priv->main_vbox));
+  res = NULL;
+  for (l = list; l; l = l->next)
+    {
+      res = g_list_append (res, cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (l->data)));
+    }
+
+  g_list_free (list);
+
+  return res;
+}
+
+static gboolean
+keynav_failed (GtkIconView        *current_view,
+               GtkDirectionType    direction,
+               GnomeControlCenter *shell)
+{
+  GList *views, *v;
+  GtkIconView *new_view;
+  GtkTreePath *path;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  gint col, c, dist, d;
+  GtkTreePath *sel;
+  gboolean res;
+
+  res = FALSE;
+
+  views = get_item_views (shell);
+
+  for (v = views; v; v = v->next)
+    {
+      if (v->data == current_view)
+        break;
+    }
+
+  if (direction == GTK_DIR_DOWN && v->next != NULL)
+    {
+      new_view = v->next->data;
+
+      if (gtk_icon_view_get_cursor (current_view, &path, NULL))
+        {
+          col = gtk_icon_view_get_item_column (current_view, path);
+          gtk_tree_path_free (path);
+
+          sel = NULL;
+          dist = 1000;
+          model = gtk_icon_view_get_model (new_view);
+          gtk_tree_model_get_iter_first (model, &iter);
+          do {
+            path = gtk_tree_model_get_path (model, &iter);
+            c = gtk_icon_view_get_item_column (new_view, path);
+            d = ABS (c - col);
+            if (d < dist)
+              {
+                if (sel)
+                  gtk_tree_path_free (sel);
+                sel = path;
+                dist = d;
+              }
+            else
+              gtk_tree_path_free (path);
+          } while (gtk_tree_model_iter_next (model, &iter));
+
+          gtk_icon_view_set_cursor (new_view, sel, NULL, FALSE);
+          gtk_tree_path_free (sel);
+        }
+
+      gtk_widget_grab_focus (GTK_WIDGET (new_view));
+
+      res = TRUE;
+    }
+
+  if (direction == GTK_DIR_UP && v->prev != NULL)
+    {
+      new_view = v->prev->data;
+
+      if (gtk_icon_view_get_cursor (current_view, &path, NULL))
+        {
+          col = gtk_icon_view_get_item_column (current_view, path);
+          gtk_tree_path_free (path);
+
+          sel = NULL;
+          dist = 1000;
+          model = gtk_icon_view_get_model (new_view);
+          gtk_tree_model_get_iter_first (model, &iter);
+          do {
+            path = gtk_tree_model_get_path (model, &iter);
+            c = gtk_icon_view_get_item_column (new_view, path);
+            d = ABS (c - col);
+            if (d <= dist)
+              {
+                if (sel)
+                  gtk_tree_path_free (sel);
+                sel = path;
+                dist = d;
+              }
+            else
+              gtk_tree_path_free (path);
+          } while (gtk_tree_model_iter_next (model, &iter));
+
+          gtk_icon_view_set_cursor (new_view, sel, NULL, FALSE);
+          gtk_tree_path_free (sel);
+        }
+
+      gtk_widget_grab_focus (GTK_WIDGET (new_view));
+
+      res = TRUE;
+    }
+
+  g_list_free (views);
+
+  return res;
+}
+
 static gboolean
 model_filter_func (GtkTreeModel              *model,
                    GtkTreeIter               *iter,
@@ -267,17 +460,22 @@ search_entry_changed_cb (GtkEntry                  *entry,
   if (!g_strcmp0 (priv->filter_string, ""))
     {
       shell_show_overview_page (priv);
+      g_object_set (G_OBJECT (entry),
+                    "secondary-icon-name", "gtk-find",
+                    "secondary-icon-activatable", FALSE,
+                    "secondary-icon-sensitive", FALSE,
+                    NULL);
     }
   else
     {
       gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (priv->search_filter));
-      gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), SEARCH_PAGE);
-
-      gtk_label_set_text (GTK_LABEL (gtk_builder_get_object (priv->builder,
-                                                             "label-title")),
-                          "");
-      gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (priv->builder,
-                                                           "title-alignment")));
+      gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
+                                     SEARCH_PAGE);
+      g_object_set (G_OBJECT (entry),
+                          "secondary-icon-name", "gtk-clear",
+                          "secondary-icon-activatable", TRUE,
+                          "secondary-icon-sensitive", TRUE,
+                          NULL);
     }
 }
 
@@ -286,7 +484,7 @@ search_entry_key_press_event_cb (GtkEntry    *entry,
                                  GdkEventKey *event,
                                  GnomeControlCenterPrivate   *priv)
 {
-  if (event->keyval == GDK_Return)
+  if (event->keyval == GDK_KEY_Return)
     {
       GtkTreePath *path;
 
@@ -300,7 +498,7 @@ search_entry_key_press_event_cb (GtkEntry    *entry,
       return TRUE;
     }
 
-  if (event->keyval == GDK_Escape)
+  if (event->keyval == GDK_KEY_Escape)
     {
       gtk_entry_set_text (entry, "");
       return TRUE;
@@ -335,8 +533,8 @@ setup_search (GnomeControlCenter *shell)
 
   /* set up the search view */
   priv->search_view = search_view = cc_shell_item_view_new ();
-  gtk_icon_view_set_orientation (GTK_ICON_VIEW (search_view),
-                                 GTK_ORIENTATION_HORIZONTAL);
+  gtk_icon_view_set_item_orientation (GTK_ICON_VIEW (search_view),
+                                      GTK_ORIENTATION_HORIZONTAL);
   gtk_icon_view_set_spacing (GTK_ICON_VIEW (search_view), 6);
   gtk_icon_view_set_model (GTK_ICON_VIEW (search_view),
                            GTK_TREE_MODEL (priv->search_filter));
@@ -380,11 +578,12 @@ fill_model (GnomeControlCenter *shell)
   GSList *list, *l;
   GMenuTreeDirectory *d;
   GMenuTree *tree;
-  GtkWidget *vbox;
 
   GnomeControlCenterPrivate *priv = shell->priv;
 
-  vbox = W (priv->builder, "main-vbox");
+  gtk_container_set_border_width (GTK_CONTAINER (shell->priv->main_vbox), 10);
+  gtk_container_set_focus_vadjustment (GTK_CONTAINER (shell->priv->main_vbox),
+                                       gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (shell->priv->scrolled_window)));
 
   tree = gmenu_tree_lookup (MENUDIR "/gnomecc.menu", 0);
 
@@ -423,11 +622,21 @@ fill_model (GnomeControlCenter *shell)
                                                   g_strdup (dir_name), g_free);
 
           categoryview = cc_shell_category_view_new (dir_name, filter);
-          gtk_box_pack_start (GTK_BOX (vbox), categoryview, FALSE, TRUE, 6);
+          gtk_box_pack_start (GTK_BOX (shell->priv->main_vbox), categoryview, FALSE, TRUE, 0);
           g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
                             "desktop-item-activated",
                             G_CALLBACK (item_activated_cb), shell);
           gtk_widget_show (categoryview);
+          g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
+                           "focus-in-event",
+                           G_CALLBACK (category_focus_in), shell);
+          g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
+                           "focus-out-event",
+                           G_CALLBACK (category_focus_out), shell);
+          g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
+                           "keynav-failed",
+                           G_CALLBACK (keynav_failed), shell);
+
 
           /* add the items from this category to the model */
           for (f = contents; f; f = f->next)
@@ -439,6 +648,8 @@ fill_model (GnomeControlCenter *shell)
                                            f->data);
                 }
             }
+
+          g_slist_free (contents);
         }
     }
 
@@ -480,7 +691,7 @@ home_button_clicked_cb (GtkButton *button,
 
 static void
 notebook_switch_page_cb (GtkNotebook               *book,
-                         GtkNotebookPage           *page,
+                         GtkWidget                 *child,
                          gint                       page_num,
                          GnomeControlCenterPrivate *priv)
 {
@@ -506,7 +717,7 @@ _shell_set_active_panel_from_id (CcShell      *shell,
   GtkTreeIter iter;
   gboolean iter_valid;
   gchar *name = NULL;
-  gchar *desktop;
+  gchar *desktop, *icon_name;
   GnomeControlCenterPrivate *priv = GNOME_CONTROL_CENTER (shell)->priv;
 
 
@@ -520,9 +731,11 @@ _shell_set_active_panel_from_id (CcShell      *shell,
 
       gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter,
                           COL_NAME, &name,
-                          COL_ID, &id,
                           COL_DESKTOP_FILE, &desktop,
+                          COL_ICON_NAME, &icon_name,
+                          COL_ID, &id,
                           -1);
+
       if (id && !strcmp (id, start_id))
         {
           g_free (id);
@@ -533,6 +746,7 @@ _shell_set_active_panel_from_id (CcShell      *shell,
           g_free (id);
           g_free (name);
           g_free (desktop);
+          g_free (icon_name);
 
           name = NULL;
           id = NULL;
@@ -549,17 +763,22 @@ _shell_set_active_panel_from_id (CcShell      *shell,
     }
   else
     {
-      activate_panel (GNOME_CONTROL_CENTER (shell), start_id, desktop);
+      activate_panel (GNOME_CONTROL_CENTER (shell), start_id, desktop, name,
+                      icon_name);
 
       g_free (name);
       g_free (desktop);
+      g_free (icon_name);
 
       return TRUE;
     }
 }
 
-
-
+static GtkWidget *
+_shell_get_toplevel (CcShell *shell)
+{
+  return GNOME_CONTROL_CENTER (shell)->priv->window;
+}
 
 /* GObject Implementation */
 static void
@@ -632,13 +851,24 @@ gnome_control_center_finalize (GObject *object)
 {
   GnomeControlCenterPrivate *priv = GNOME_CONTROL_CENTER (object)->priv;
 
-
   if (priv->filter_string)
     {
       g_free (priv->filter_string);
       priv->filter_string = NULL;
     }
 
+  if (priv->default_window_title)
+    {
+      g_free (priv->default_window_title);
+      priv->default_window_title = NULL;
+    }
+
+  if (priv->default_window_icon)
+    {
+      g_free (priv->default_window_icon);
+      priv->default_window_icon = NULL;
+    }
+
   G_OBJECT_CLASS (gnome_control_center_parent_class)->finalize (object);
 }
 
@@ -656,13 +886,48 @@ gnome_control_center_class_init (GnomeControlCenterClass *klass)
   object_class->finalize = gnome_control_center_finalize;
 
   shell_class->set_active_panel_from_id = _shell_set_active_panel_from_id;
+  shell_class->get_toplevel = _shell_get_toplevel;
+}
+
+static void
+on_window_size_allocate (GtkWidget          *widget,
+                         GtkAllocation      *allocation,
+                         GnomeControlCenter *self)
+{
+  int height;
+
+  if (gtk_notebook_get_current_page (GTK_NOTEBOOK (self->priv->notebook)) == OVERVIEW_PAGE)
+    {
+      gtk_widget_get_preferred_height_for_width (GTK_WIDGET (self->priv->main_vbox),
+                                                 FIXED_WIDTH,
+                                                 NULL,
+                                                 &height);
+      if (gtk_widget_get_realized (widget))
+        {
+          int          monitor;
+          GdkScreen   *screen;
+          GdkRectangle rect;
+          GdkWindow   *window;
+
+          window = gtk_widget_get_window (widget);
+          screen = gtk_widget_get_screen (widget);
+          monitor = gdk_screen_get_monitor_at_window (screen, window);
+          gdk_screen_get_monitor_geometry (screen, monitor, &rect);
+          height = MIN (height + 10, rect.height);
+        }
+    }
+  else
+    {
+      height = 50;
+    }
+
+  gtk_scrolled_window_set_min_content_height (GTK_SCROLLED_WINDOW (self->priv->scrolled_window), height);
 }
 
 static void
 gnome_control_center_init (GnomeControlCenter *self)
 {
   GError *err = NULL;
-  GtkWidget *vbox;
   GnomeControlCenterPrivate *priv;
 
   priv = self->priv = CONTROL_CENTER_PRIVATE (self);
@@ -680,27 +945,19 @@ gnome_control_center_init (GnomeControlCenter *self)
 
   /* connect various signals */
   priv->window = W (priv->builder, "main-window");
-  g_signal_connect (priv->window, "delete-event", G_CALLBACK (gtk_main_quit),
-                    NULL);
+  g_signal_connect_swapped (priv->window, "delete-event", G_CALLBACK (g_object_unref), self);
 
   priv->notebook = W (priv->builder, "notebook");
-
+  priv->scrolled_window = W (priv->builder, "scrolledwindow1");
+  gtk_widget_set_size_request (priv->scrolled_window, FIXED_WIDTH, -1);
+  priv->main_vbox = W (priv->builder, "main-vbox");
+  g_signal_connect (priv->scrolled_window, "size-allocate", G_CALLBACK (on_window_size_allocate), self);
   g_signal_connect (priv->notebook, "switch-page",
                     G_CALLBACK (notebook_switch_page_cb), priv);
 
   g_signal_connect (gtk_builder_get_object (priv->builder, "home-button"),
                     "clicked", G_CALLBACK (home_button_clicked_cb), self);
 
-
-  /* setup background colours */
-  vbox = W (priv->builder, "main-vbox");
-  gtk_widget_set_size_request (vbox, 0, -1);
-
-  gtk_widget_modify_bg (vbox->parent, GTK_STATE_NORMAL,
-                        &vbox->style->base[GTK_STATE_NORMAL]);
-  gtk_widget_modify_fg (vbox->parent, GTK_STATE_NORMAL,
-                        &vbox->style->text[GTK_STATE_NORMAL]);
-
   /* load the available settings panels */
   fill_model (self);
 
@@ -710,7 +967,9 @@ gnome_control_center_init (GnomeControlCenter *self)
   /* setup search functionality */
   setup_search (self);
 
-  gtk_widget_show_all (priv->window);
+  /* store default window title and name */
+  priv->default_window_title = g_strdup (gtk_window_get_title (GTK_WINDOW (priv->window)));
+  priv->default_window_icon = g_strdup (gtk_window_get_icon_name (GTK_WINDOW (priv->window)));
 }
 
 GnomeControlCenter *
@@ -724,3 +983,11 @@ gnome_control_center_present (GnomeControlCenter *center)
 {
   gtk_window_present (GTK_WINDOW (center->priv->window));
 }
+
+void
+gnome_control_center_show (GnomeControlCenter *center,
+			   GtkApplication     *app)
+{
+  gtk_window_set_application (GTK_WINDOW (center->priv->window), app);
+  gtk_widget_show_all (center->priv->window);
+}
